@@ -2,15 +2,16 @@ use std::task::{Context, Poll};
 use std::time::{Duration, Instant};
 use std::{cell::RefCell, collections::VecDeque, future::Future, pin::Pin, rc::Rc};
 
-use h2::client::{handshake, Connection, SendRequest};
+use h2::client::{Builder, Connection, SendRequest};
 use http::uri::Authority;
 
 use crate::channel::pool;
 use crate::codec::{AsyncRead, AsyncWrite, ReadBuf};
 use crate::http::Protocol;
-use crate::rt::{spawn, time::sleep, time::Sleep};
+use crate::rt::spawn;
 use crate::service::Service;
 use crate::task::LocalWaker;
+use crate::time::{now, sleep, Millis, Sleep};
 use crate::util::{poll_fn, Bytes, HashMap};
 
 use super::connection::{ConnectionType, IoConnection};
@@ -30,7 +31,6 @@ impl From<Authority> for Key {
 
 type Waiter<Io> = pool::Sender<Result<IoConnection<Io>, ConnectError>>;
 type WaiterReceiver<Io> = pool::Receiver<Result<IoConnection<Io>, ConnectError>>;
-const ZERO: Duration = Duration::from_millis(0);
 
 /// Connections pool
 pub(super) struct ConnectionPool<T, Io: 'static>(Rc<T>, Rc<RefCell<Inner<Io>>>);
@@ -47,7 +47,7 @@ where
         connector: T,
         conn_lifetime: Duration,
         conn_keep_alive: Duration,
-        disconnect_timeout: Duration,
+        disconnect_timeout: Millis,
         limit: usize,
     ) -> Self {
         let connector = Rc::new(connector);
@@ -180,7 +180,7 @@ struct AvailableConnection<Io> {
 pub(super) struct Inner<Io> {
     conn_lifetime: Duration,
     conn_keep_alive: Duration,
-    disconnect_timeout: Duration,
+    disconnect_timeout: Millis,
     limit: usize,
     acquired: usize,
     available: HashMap<Key, VecDeque<AvailableConnection<Io>>>,
@@ -239,7 +239,7 @@ where
         // check if open connection is available
         // cleanup stale connections at the same time
         if let Some(ref mut connections) = self.available.get_mut(key) {
-            let now = Instant::now();
+            let now = now();
             while let Some(conn) = connections.pop_back() {
                 // check if it still usable
                 if (now - conn.used) > self.conn_keep_alive
@@ -279,7 +279,7 @@ where
             .push_back(AvailableConnection {
                 io,
                 created,
-                used: Instant::now(),
+                used: now(),
             });
         self.check_availibility();
     }
@@ -363,29 +363,21 @@ where
     }
 }
 
-pin_project_lite::pin_project! {
-    struct CloseConnection<T> {
-        io: T,
-        #[pin]
-        timeout: Option<Sleep>,
-        shutdown: bool,
-    }
+struct CloseConnection<T> {
+    io: T,
+    timeout: Option<Sleep>,
+    shutdown: bool,
 }
 
 impl<T> CloseConnection<T>
 where
     T: AsyncWrite + AsyncRead + Unpin + 'static,
 {
-    fn spawn(io: T, timeout: Duration) {
-        let timeout = if timeout != ZERO {
-            Some(sleep(timeout))
-        } else {
-            None
-        };
+    fn spawn(io: T, timeout: Millis) {
         spawn(Self {
             io,
-            timeout,
             shutdown: false,
+            timeout: timeout.map(sleep),
         });
     }
 }
@@ -396,19 +388,19 @@ where
 {
     type Output = ();
 
-    fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<()> {
-        let mut this = self.project();
+    fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<()> {
+        let mut this = self.as_mut();
 
         // shutdown WRITE side
         match Pin::new(&mut this.io).poll_shutdown(cx) {
-            Poll::Ready(Ok(())) => *this.shutdown = true,
+            Poll::Ready(Ok(())) => this.shutdown = true,
             Poll::Pending => return Poll::Pending,
             Poll::Ready(Err(_)) => return Poll::Ready(()),
         }
 
         // read until 0 or err
-        if let Some(timeout) = this.timeout.as_pin_mut() {
-            match timeout.poll(cx) {
+        if let Some(ref timeout) = this.timeout {
+            match timeout.poll_elapsed(cx) {
                 Poll::Ready(_) => (),
                 Poll::Pending => {
                     let mut buf = [0u8; 512];
@@ -488,7 +480,7 @@ where
                     // h2 connection is ready
                     let conn = IoConnection::new(
                         ConnectionType::H2(snd),
-                        Instant::now(),
+                        now(),
                         Some(this.guard.take().unwrap().consume()),
                     );
                     if let Err(Ok(conn)) = this.tx.take().unwrap().send(Ok(conn)) {
@@ -525,7 +517,7 @@ where
                 if proto == Protocol::Http1 {
                     let conn = IoConnection::new(
                         ConnectionType::H1(io),
-                        Instant::now(),
+                        now(),
                         Some(this.guard.take().unwrap().consume()),
                     );
                     if let Err(Ok(conn)) = this.tx.take().unwrap().send(Ok(conn)) {
@@ -535,7 +527,7 @@ where
                     Poll::Ready(())
                 } else {
                     // init http2 handshake
-                    this.h2 = Some(Box::pin(handshake(io)));
+                    this.h2 = Some(Box::pin(Builder::new().handshake(io)));
                     self.poll(cx)
                 }
             }
@@ -608,10 +600,9 @@ impl<T> Drop for Acquired<T> {
 
 #[cfg(test)]
 mod tests {
-    use std::{cell::RefCell, convert::TryFrom, rc::Rc, time::Duration};
+    use std::{cell::RefCell, convert::TryFrom, rc::Rc};
 
     use super::*;
-    use crate::rt::time::sleep;
     use crate::{
         http::client::Connection, http::Uri, service::fn_service, testing::Io,
         util::lazy,
@@ -630,7 +621,7 @@ mod tests {
             }),
             Duration::from_secs(10),
             Duration::from_secs(10),
-            Duration::from_millis(0),
+            Millis::ZERO,
             1,
         )
         .clone();
@@ -671,7 +662,7 @@ mod tests {
         let mut fut = pool.call(req.clone());
         assert!(lazy(|cx| Pin::new(&mut fut).poll(cx)).await.is_pending());
         drop(fut);
-        sleep(Duration::from_millis(50)).await;
+        sleep(Millis(50)).await;
         pool.1.borrow_mut().check_availibility();
         assert!(pool.1.borrow().waiters.is_empty());
 
