@@ -3,13 +3,14 @@ use std::{
     task::Poll,
 };
 
-use bytes::Bytes;
 use h2::server::{self, Handshake};
 
 use crate::codec::{AsyncRead, AsyncWrite};
 use crate::framed::State;
 use crate::rt::net::TcpStream;
 use crate::service::{pipeline_factory, IntoServiceFactory, Service, ServiceFactory};
+use crate::time::{Millis, Seconds};
+use crate::util::Bytes;
 
 use super::body::MessageBody;
 use super::builder::HttpServiceBuilder;
@@ -60,10 +61,10 @@ where
     /// Create new `HttpService` instance.
     pub fn new<F: IntoServiceFactory<S>>(service: F) -> Self {
         let cfg = ServiceConfig::new(
-            KeepAlive::Timeout(5),
-            5000,
-            0,
-            5000,
+            KeepAlive::Timeout(Seconds(5)),
+            Millis(5_000),
+            Seconds::ZERO,
+            Millis(5_000),
             1024,
             8 * 1024,
             8 * 1024,
@@ -257,7 +258,7 @@ mod openssl {
         > {
             pipeline_factory(
                 Acceptor::new(acceptor)
-                    .timeout((self.cfg.0.ssl_handshake_timeout as u64) * 1000)
+                    .timeout(self.cfg.0.ssl_handshake_timeout)
                     .map_err(SslError::Ssl)
                     .map_init_err(|_| panic!()),
             )
@@ -282,7 +283,7 @@ mod openssl {
 #[cfg(feature = "rustls")]
 mod rustls {
     use super::*;
-    use crate::server::rustls::{Acceptor, ServerConfig, Session, TlsStream};
+    use crate::server::rustls::{Acceptor, ServerConfig, TlsStream};
     use crate::server::SslError;
 
     impl<S, B, X, U> HttpService<TlsStream<TcpStream>, S, B, X, U>
@@ -321,11 +322,11 @@ mod rustls {
             InitError = (),
         > {
             let protos = vec!["h2".to_string().into(), "http/1.1".to_string().into()];
-            config.set_protocols(&protos);
+            config.alpn_protocols = protos;
 
             pipeline_factory(
                 Acceptor::new(config)
-                    .timeout((self.cfg.0.ssl_handshake_timeout as u64) * 1000)
+                    .timeout(self.cfg.0.ssl_handshake_timeout)
                     .map_err(SslError::Ssl)
                     .map_init_err(|_| panic!()),
             )
@@ -333,7 +334,7 @@ mod rustls {
                 let proto = io
                     .get_ref()
                     .1
-                    .get_alpn_protocol()
+                    .alpn_protocol()
                     .and_then(|protos| {
                         if protos.windows(2).any(|window| window == b"h2") {
                             Some(Protocol::Http2)
@@ -515,7 +516,7 @@ where
             Protocol::Http2 => HttpServiceHandlerResponse {
                 state: ResponseState::H2Handshake {
                     data: Some((
-                        server::handshake(io),
+                        server::Builder::new().handshake(io),
                         self.config.clone(),
                         on_connect,
                         peer_addr,
